'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = initializeApollo;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _apolloClient = require('apollo-client');

var _apolloClient2 = _interopRequireDefault(_apolloClient);

var _reactApollo = require('react-apollo');

var _graphqlTools = require('graphql-tools');

var _graphql = require('graphql');

var _apolloCacheInmemory = require('apollo-cache-inmemory');

var _apolloLink = require('apollo-link');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Delay for mock latency
 */
function delay(ms) {
  return new Promise(function (resolve) {
    setTimeout(function () {
      resolve();
    }, ms);
  });
}

/**
 * Create an Apollo Link
 */
function createLink(schema) {
  var rootValue = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var context = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};

  var delayMs = options && options.delayMs || 300;
  return new _apolloLink.ApolloLink(function (operation) {
    return new _apolloLink.Observable(function (observer) {
      var query = operation.query,
          operationName = operation.operationName,
          variables = operation.variables;

      delay(delayMs).then(function () {
        return (0, _graphql.graphql)(schema, (0, _graphql.print)(query), rootValue, context, variables, operationName);
      }).then(function (result) {
        observer.next(result);
        observer.complete();
      }).catch(observer.error.bind(observer));
    });
  });
}

function initializeApollo(_ref) {
  var typeDefs = _ref.typeDefs,
      mocks = _ref.mocks,
      _ref$apolloLinkOption = _ref.apolloLinkOptions,
      apolloLinkOptions = _ref$apolloLinkOption === undefined ? {} : _ref$apolloLinkOption,
      _ref$apolloClientOpti = _ref.apolloClientOptions,
      apolloClientOptions = _ref$apolloClientOpti === undefined ? {} : _ref$apolloClientOpti,
      typeResolvers = _ref.typeResolvers,
      _ref$context = _ref.context,
      context = _ref$context === undefined ? {} : _ref$context,
      _ref$rootValue = _ref.rootValue,
      rootValue = _ref$rootValue === undefined ? {} : _ref$rootValue,
      _ref$cacheOptions = _ref.cacheOptions,
      cacheOptions = _ref$cacheOptions === undefined ? {} : _ref$cacheOptions;

  var schema = (0, _graphqlTools.makeExecutableSchema)({ typeDefs: typeDefs });

  if (!!mocks) {
    (0, _graphqlTools.addMockFunctionsToSchema)({
      schema: schema,
      mocks: mocks
    });
  }

  if (!!typeResolvers) {
    (0, _graphqlTools.addResolveFunctionsToSchema)(schema, typeResolvers);
  }

  var graphqlClient = new _apolloClient2.default(_extends({
    addTypename: true,
    cache: new _apolloCacheInmemory.InMemoryCache(cacheOptions),
    link: createLink(schema, rootValue, context, apolloLinkOptions),
    connectToDevTools: true
  }, apolloClientOptions));

  function StorybookProvider(_ref2) {
    var children = _ref2.children;

    return _react2.default.createElement(
      _reactApollo.ApolloProvider,
      { client: graphqlClient },
      _react2.default.createElement(
        'div',
        null,
        children
      )
    );
  }

  return function (story) {
    return _react2.default.createElement(
      StorybookProvider,
      null,
      story()
    );
  };
}